@@ -411,65 +411,6 @@ fn verify_all_signatures() {
                 "__smusdx",
                 "__usad8",
                 "__usada8",
-                "vld1_s8",
-                "vld1q_s8",
-                "vld1q_s8",
-                "vld1_s16",
-                "vld1q_s16",
-                "vld1_s32",
-                "vld1q_s32",
-                "vld1_s64",
-                "vld1q_s64",
-                "vld1_u8",
-                "vld1q_u8",
-                "vld1_u16",
-                "vld1q_u16",
-                "vld1_u32",
-                "vld1q_u32",
-                "vld1_u64",
-                "vld1q_u64",
-                "vld1_p8",
-                "vld1q_p8",
-                "vld1_p16",
-                "vld1q_p16",
-                "vld1_f32",
-                "vld1q_f32",
-                "vld1_f64",
-                "vld1q_f64",
-                "vst1_s8",
-                "vst1q_s8",
-                "vst1_s16",
-                "vst1q_s16",
-                "vst1_s32",
-                "vst1q_s32",
-                "vst1_s64",
-                "vst1q_s64",
-                "vst1_u8",
-                "vst1q_u8",
-                "vst1_u16",
-                "vst1q_u16",
-                "vst1_u32",
-                "vst1q_u32",
-                "vst1_u64",
-                "vst1q_u64",
-                "vst1_p8",
-                "vst1q_p8",
-                "vst1_p16",
-                "vst1q_p16",
-                "vst1_f32",
-                "vst1q_f32",
-                "vpadal_s8",
-                "vpadal_s16",
-                "vpadal_s32",
-                "vpadalq_s8",
-                "vpadalq_s16",
-                "vpadalq_s32",
-                "vpadal_u8",
-                "vpadal_u16",
-                "vpadal_u32",
-                "vpadalq_u8",
-                "vpadalq_u16",
-                "vpadalq_u32",
                 "__ldrex",
                 "__strex",
                 "__ldrexb",
@@ -515,12 +456,36 @@ fn verify_all_signatures() {
             "vqrdmlahh_laneq_s16",
             "vqrdmlahs_lane_s32",
             "vqrdmlahs_laneq_s32",
+            "vqrdmlah_s16",
+            "vqrdmlah_s32",
+            "vqrdmlahq_s16",
+            "vqrdmlahq_s32",
+            "vqrdmlah_lane_s16",
+            "vqrdmlah_laneq_s16",
+            "vqrdmlahq_lane_s16",
+            "vqrdmlahq_laneq_s16",
+            "vqrdmlah_lane_s32",
+            "vqrdmlah_laneq_s32",
+            "vqrdmlahq_lane_s32",
+            "vqrdmlahq_laneq_s32",
             "vqrdmlshh_s16",
             "vqrdmlshs_s32",
             "vqrdmlshh_lane_s16",
             "vqrdmlshh_laneq_s16",
             "vqrdmlshs_lane_s32",
             "vqrdmlshs_laneq_s32",
+            "vqrdmlsh_s16",
+            "vqrdmlshq_s16",
+            "vqrdmlsh_s32",
+            "vqrdmlshq_s32",
+            "vqrdmlsh_lane_s16",
+            "vqrdmlsh_laneq_s16",
+            "vqrdmlshq_lane_s16",
+            "vqrdmlshq_laneq_s16",
+            "vqrdmlsh_lane_s32",
+            "vqrdmlsh_laneq_s32",
+            "vqrdmlshq_lane_s32",
+            "vqrdmlshq_laneq_s32",
             "__dbg",
         ];
         let arm = match map.get(rust.name) {
